#
# Reports dead links in the table of contents used by docfx and .md files that aren't linked anywhere in the TOC
#

$expectedMissingLinks = '.\implemented-views.md' #Implemented views is generated by the CI build
$expectedUnlinked = '.\roadmap.md'
$tempFileName = 'toc_additions.yml.tmp'

$allMdFiles = Get-ChildItem -Recurse -Filter *.md | Resolve-Path -Relative | Where-Object {$_.IndexOf('\implemented\') -lt 0}
$toc = Get-Content -Path .\toc.yml 
$allMdLinks = $toc | foreach {$_ -split ":"} | foreach {$_.Trim(' ')} | foreach {'.\' + $_} | foreach {$_ -replace '/', '\'} | Where-Object {$_.EndsWith(".md")}
$badLinks = $allMdLinks | Where-Object {-not (Test-Path -Path $_)} | Where-Object {-not ($expectedMissingLinks -contains $_)}
$unlinkedFiles = $allMdFiles | Where-Object {-not ($allMdLinks -contains $_)} | Where-Object {-not ($expectedUnlinked -contains $_)}
if ($badLinks.Length -gt 0) {
	Write-Host "Bad links found in toc.yml: $badLinks" -ForegroundColor Red
}
else {
	Write-Host 'No bad links found in toc.yml'
}
if ($unlinkedFiles.Length -gt 0) {
	Write-Host ".md files not linked in toc.yml: $unlinkedFiles"
	'# UNLINKED .MD FILES: Add to toc.yml in appropriate category' | Out-File $tempFileName
	$unlinkedFiles | foreach {
		$header = Get-Content $_ | Where-Object {$_.StartsWith('#')}[0] | foreach {$_.Trim('#').Trim(' ')} | Select -First 1
		"    - name: $header" | Out-File $tempFileName -Append
		$link = $_.Trim('.').Trim('\')
		"      href: $link" | Out-File $tempFileName -Append
	}
	Write-Host "Missing links added to $tempFileName" -ForegroundColor Green
}
else {
	Write-Host 'No unlinked files found'
	if (Test-Path $tempFileName) {
		'# No unlinked .md files' | Out-File $tempFileName
	}
}
