using System;
using System.ComponentModel;
using System.Globalization;

namespace Windows.UI.Xaml
{
	[TypeConverter(typeof(ThicknessConverter))]
	public partial struct Thickness : IEquatable<Thickness>
	{
		public static readonly Thickness Empty = new Thickness ();

		public Thickness (double uniformLength)
			: this ()
		{
			Left = Top = Right = Bottom = uniformLength;
		}

		public Thickness (double left, double top, double right, double bottom)
			: this ()
		{
			Left = left;
			Top = top;
			Right = right;
			Bottom = bottom;
		}

		public Thickness (double leftRight, double topBottom)
			: this ()
		{
			Left = leftRight;
			Top = topBottom;
			Right = leftRight;
			Bottom = topBottom;
		}

		public double Left { get ; set; }

		public double Top { get ; set; }

		public double Right { get ; set; }

		public double Bottom { get ; set; }

		public bool Equals (Thickness other)
		{
			return  Math.Abs (Left - other.Left) < double.Epsilon
			&& Math.Abs (Top - other.Top) < double.Epsilon
			&& Math.Abs (Right - other.Right) < double.Epsilon
			&& Math.Abs (Bottom - other.Bottom) < double.Epsilon;
		}

		public override bool Equals (object obj)
		{
			return (obj is Thickness) && Equals ((Thickness)obj);
		}

		public override int GetHashCode ()
		{
			// Analysis disable NonReadonlyReferencedInGetHashCode
			return Left.GetHashCode ()
			^ Top.GetHashCode ()
			^ Right.GetHashCode ()
			^ Bottom.GetHashCode ();
			// Analysis restore NonReadonlyReferencedInGetHashCode
		}

		public override string ToString ()
		{
			return string.Format (CultureInfo.InvariantCulture, "[Thickness: {0}-{1}-{2}-{3}]", Left, Top, Right, Bottom);
		}

		public static bool operator == (Thickness t1, Thickness t2)
		{
			return t1.Equals (t2);
		}

		public static bool operator != (Thickness t1, Thickness t2)
		{
			return !t1.Equals (t2);
		}
	}
}

